include("Civ6Common");
include("SupportFunctions");

-- // ------------------------------------------------
-- // Constants
-- // ------------------------------------------------
local LEADER_HANNO:string = "LEADER_GT_HANNO";

-- // ------------------------------------------------
-- // Variables
-- // ------------------------------------------------
local m_IsLoading:boolean = false;
local m_NavalTechnologiesResearched:boolean = false;

-- // ------------------------------------------------
-- // Functions
-- // ------------------------------------------------
function IsPlayerLeader(playerID:number, leaderType:string)

	local result:boolean = false;

	local playerConfig:table = PlayerConfigurations[playerID];

	if (playerConfig:GetLeaderTypeName() == leaderType) then
		result = true;
	end

	return result;
end

function GetPlayerByID(playerID:number)

	local player:table = {};

	for i, major in ipairs(PlayerManager:GetAliveMajors()) do
		if (major:GetID() == playerID) then
			player = major;
			break;
		end
	end

	return player;
end

function SpawnNavalUnit(playerID:number, techID:number)

	print("SpawnNavalUnit: playerID = " .. playerID);

	local player:table = GetPlayerByID(playerID);
	local technologyType:string = GameInfo.Technologies[techID].TechnologyType;
	local plotX, plotY = GetCapitalPlot(player);

	if (plotX == -1 or plotY == -1) then
		-- no valid place to spawn the unit
		print("SpawnNavalUnit: No valid plot to spawn unit at!");
		return;
	end

	if (technologyType ~= nil) then
		print("SpawnNavalUnit: technologyType = " .. technologyType);
	
		if (technologyType == "TECH_SAILING") then
			player:GetUnits():Create(GameInfo.Units["UNIT_GALLEY"].Index, plotX, plotY);
		end

		if (technologyType == "TECH_SHIPBUILDING") then
			player:GetUnits():Create(GameInfo.Units["UNIT_GT_QUINQUEREME"].Index, plotX, plotY);
		end

		if (technologyType == "TECH_CELESTIAL_NAVIGATION") then
			player:GetUnits():Create(GameInfo.Units["UNIT_GT_QUINQUEREME"].Index, plotX, plotY);
		end

		if (technologyType == "TECH_CARTOGRAPHY") then
			player:GetUnits():Create(GameInfo.Units["UNIT_CARAVEL"].Index, plotX, plotY);
		end

		if (technologyType == "TECH_SQUARE_RIGGING") then
			player:GetUnits():Create(GameInfo.Units["UNIT_FRIGATE"].Index, plotX, plotY);
			m_NavalTechnologiesResearched = true;
		end
	else
		print("SpawnNavalUnit: technologyType is nil!");
	end

end

function GetCapitalPlot(player:table)

	local X:number = -1;
	local Y:number = -1;

	if (player ~= nil) then
		local playerCities:table = player:GetCities();

		if (playerCities ~= nil) then
			local playerCapital:table = playerCities:GetCapitalCity();

			if (playerCapital ~= nil) then
				X = playerCapital:GetX();
				Y = playerCapital:GetY();
			end
		end
	end

	return X, Y;
end

-- GetCapitalHarborPlot does not work as the call to City:GetDistricts():Members() always throws an error
-- even though I check for nil. Think this may have something to do with the Gameplay context as examples
-- in the Firaxis code are all in the UI context.

--function GetCapitalHarborPlot(player:table)
--
--	local X:number = -1;
--	local Y:number = -1;
--
--	if (player ~= nil) then
--		local playerCities:table = player:GetCities();
--
--		if (playerCities ~= nil) then
--			local playerCapital:table = playerCities:GetCapitalCity();
--
--			if (playerCapital ~= nil) then
--				print("GetCapitalHarborPlot: Capital found. Calling City:GetDistricts ...");
--
--				local hasHarbor:boolean = false;
--				local cityDistricts:table = playerCapital:GetDistricts();
--
--				if (cityDistricts ~= nil) then
--					print("GetCapitalHarborPlot: Checking capital districts collection ...");
--
--					-- does the city have a harbor? what is it's location?
--					for i, district in cityDistricts:Members() do
--						print("GetCapitalHarborPlot: Getting districtInfo data ...");
--						local districtInfo:table = GameInfo.Districts[district:GetType()];
--
--						if (districtInfo ~= nil) then
--							if (districtInfo.DistrictType == "DISTRICT_HARBOR") then
--								X = district:GetX();
--								Y = district:GetY();
--
--								print("GetCapitalHarborPlot: Harbor plot found!");
--								hasHarbor = true;
--							end
--						end
--					end
--				end
--
--				if (hasHarbor == false) then
--					-- no harbor. is the city on the coast?
--					local plot:table = playerCapital:GetLocation();
--
--					if (plot.IsCoastalLand) then
--						X = plot:GetX();
--						Y = plot:GetY();
--
--						print("GetCapitalHarborPlot: No harbor but is on coast!");
--					else
--						-- not on coast. can not create naval unit
--						print("GetCapitalHarborPlot: No harbor and not on coast!");
--					end
--				end
--			end
--		end
--	end
--
--	return X, Y;
--end

-- // ------------------------------------------------
-- // Event Handlers
-- // ------------------------------------------------
function OnLoadGameViewStateDone()

	m_IsLoading = false;
	LuaEvents.UpdateCarthageTradePanel();

end

function OnLocalPlayerTurnBegin()

	LuaEvents.UpdateCarthageTradePanel();

end

function OnResearchCompleted(playerID:number, techID:number)

	if (m_IsLoading == false and m_NavalTechnologiesResearched == false) then
		if (IsPlayerLeader(playerID, LEADER_HANNO)) then
		
			SpawnNavalUnit(playerID, techID);

		end
	end

end

function OnCivicChanged(playerID:number, civicID:number)

	LuaEvents.UpdateCarthageTradePanel();

end

-- // ------------------------------------------------
-- // Init
-- // ------------------------------------------------
function Initialize()

	m_IsLoading = true;

	Events.LocalPlayerTurnBegin.Add(OnLocalPlayerTurnBegin);
	Events.LoadGameViewStateDone.Add(OnLoadGameViewStateDone);
	Events.ResearchCompleted.Add(OnResearchCompleted);
	Events.CivicChanged.Add(OnCivicChanged);

	print("CarthageGameplay.lua initialized!");
	
end
Initialize();