include("Civ6Common");
include("SupportFunctions");
include("InstanceManager");

-- // -----------------------------------------------
-- // Variables
-- // -----------------------------------------------
local m_IsLoading:boolean = false;
local m_IsAttached:boolean = false;
local m_LocalPlayer:table = {};

-- // -----------------------------------------------
-- // Functions
-- // -----------------------------------------------
function AttachPanelToWorldTracker()

	if (m_IsLoading) then
		-- the ContextPtr:LookUpControl does not function correctly if the UI is still loading
		print("CarthageUITradePanel_AttachPanelToWorldTracker: Failed! UI loading!");
		return;
	end

	if (not m_IsAttached) then
		print("CarthageUITradePanel_AttachPanelToWorldTracker: CarthageTradePanel not attached!");

		-- because the panel has not already been attached we try to get the worldtracker stack panel
		local worldTrackerPanel:table = ContextPtr:LookUpControl("/InGame/WorldTracker/PanelStack");
	
		if (worldTrackerPanel ~= nil) then
			-- attach the trade panel to the worldtracker stack panel
			Controls.CarthageTradePanel:ChangeParent(worldTrackerPanel);
			
			-- reposition the trade panel just after the civics panel in the stack
			worldTrackerPanel:AddChildAtIndex(Controls.CarthageTradePanel, 4);
		
			-- recalculate the worldtracker stack
			worldTrackerPanel:CalculateSize();
			worldTrackerPanel:ReprocessAnchoring();
			
			-- we only want to attach the panel once
			m_IsAttached = true;

			print("CarthageUITradePanel_AttachPanelToWorldTracker: CarthageTradePanel is attached!");
		end
	end

end

function UpdatePanel()

	Controls.CarthageTradeBonusLabel:SetText(Locale.Lookup("LOC_TRADE_PANEL_LABEL_PREFIX") .. "   " .. Locale.Lookup("LOC_TRADE_PANEL_LABEL_NO_BONUS"));
	Controls.CarthageTradeBonusLabel:SetColor("255,255,255,100");
	Controls.CarthageTradeBonusLabel:SetToolTipString(Locale.Lookup("LOC_CARTHAGE_NO_POLICY_BONUS_TOOLTIP"));

	if (not m_IsLoading) then
		local playerCulture:table = m_LocalPlayer:GetCulture();
		local currentCivicID:number = playerCulture:GetProgressingCivic();

		local bonusText:string = Locale.Lookup("LOC_TRADE_PANEL_LABEL_PREFIX") .. "   " .. Locale.Lookup("LOC_TRADE_PANEL_LABEL_TRADE_ROUTE_BONUS");
		local tooltipText:string = "";

		print("CarthageUITradePanel_UpdatePanel: currentCivicID = " .. currentCivicID);

		if (currentCivicID >= 0) then
			local updateLabel:boolean = false;

			if (currentCivicID == GameInfo.Civics["CIVIC_FOREIGN_TRADE"].Index) then
				tooltipText = Locale.Lookup("LOC_CARTHAGE_FOREIGN_TRADE_POLICY_TOOLTIP");
				updateLabel = true;
			elseif (currentCivicID == GameInfo.Civics["CIVIC_MERCENARIES"].Index) then
				tooltipText = Locale.Lookup("LOC_CARTHAGE_MERCENARIES_POLICY_TOOLTIP");
				updateLabel = true;
			elseif (currentCivicID == GameInfo.Civics["CIVIC_GUILDS"].Index) then
				tooltipText = Locale.Lookup("LOC_CARTHAGE_GUILDS_POLICY_TOOLTIP");
				updateLabel = true;
			elseif (currentCivicID == GameInfo.Civics["CIVIC_THE_ENLIGHTENMENT"].Index) then
				tooltipText = Locale.Lookup("LOC_CARTHAGE_THE_ENLIGHTENMENT_POLICY_TOOLTIP");
				updateLabel = true;
			elseif (currentCivicID == GameInfo.Civics["CIVIC_MERCANTILISM"].Index) then
				tooltipText = Locale.Lookup("LOC_CARTHAGE_MERCANTILISM_POLICY_TOOLTIP");
				updateLabel = true;
			elseif (currentCivicID == GameInfo.Civics["CIVIC_CAPITALISM"].Index) then
				tooltipText = Locale.Lookup("LOC_CARTHAGE_CAPITALISM_POLICY_TOOLTIP");
				updateLabel = true;
			elseif (currentCivicID == GameInfo.Civics["CIVIC_SUFFRAGE"].Index) then
				tooltipText = Locale.Lookup("LOC_CARTHAGE_SUFFRAGE_POLICY_TOOLTIP");
				updateLabel = true;
			elseif (currentCivicID == GameInfo.Civics["CIVIC_CLASS_STRUGGLE"].Index) then
				tooltipText = Locale.Lookup("LOC_CARTHAGE_CLASS_STRUGGLE_POLICY_TOOLTIP");
				updateLabel = true;
			end

			if (updateLabel) then
				Controls.CarthageTradeBonusLabel:SetText(bonusText);
				Controls.CarthageTradeBonusLabel:SetColor("255,255,255,200");
				Controls.CarthageTradeBonusLabel:SetToolTipString(tooltipText);
			end
		end
	end

end

-- // -----------------------------------------------
-- // Event Handlers
-- // -----------------------------------------------
function OnLoadGameViewStateDone()

	AttachPanelToWorldTracker();

end

function OnUpdateCarthageTradePanel()

	UpdatePanel();

end

-- // -----------------------------------------------
-- // Init
-- // -----------------------------------------------
function Initialize()

	m_IsLoading = true;

	m_LocalPlayer = Players[Game.GetLocalPlayer()];

	Events.LoadGameViewStateDone.Add(OnLoadGameViewStateDone);
	LuaEvents.UpdateCarthageTradePanel.Add(OnUpdateCarthageTradePanel);

	m_IsLoading = false;

	print("CarthageUITradePanel initialized!");

end
Initialize();